def _get_text(d: dict) -> str:
    tdict = d['body_text']
    return ' '.join([x['text'] for x in tdict])

def get_texts(l: list) -> list:
    return [_get_text(x) for x in l]

def _get_abstract(d: dict) -> str:
    l = d['abstract']
    out = '' if len(l) == 0 else l[0]['text']
    return out

def get_abstracts(l: list) -> list:
    return [_get_abstract(x) for x in l]





import json
import os
from urllib.request import urlopen
import tarfile
from typing import Callable
from .text import _get_text, _get_abstract

class Paperset:
    def __init__(self, directory: str) -> None:
        """
        The Paperset class:
            __init__ args:
                directory: a string, the directory where the jsons are stored

            description:
                lazy loader for cord-19 text files. Data is not actually loaded
                until indexing, until then it just indexes files. Can be
                indexed with both ints and slices.
        """
        self.directory = directory
        self.dir_dict = {idx: f for idx, f in enumerate(os.listdir(self.directory))}


    def _load_file(self, path: str) -> dict:
        with open(f"{self.directory}/{path}") as handle:
            outdict = json.loads(handle.read())
        return outdict


    def __getitem__(self, indices: int) -> list:
        slicedkeys = list(self.dir_dict.keys())[indices]
        if not isinstance(slicedkeys, list):
            slicedkeys=[slicedkeys]
        out =  [self._load_file(self.dir_dict[key]) for key in slicedkeys]
        if len(out) == 1:
            return out[0]
        else:
            return out

    def apply(self, fn: Callable) -> list:
        return [fn(self._load_file(self.dir_dict[k])) for k in self.dir_dict.keys()]

    def texts(self) -> list:
        return self.apply(_get_text)

    def abstracts(self) -> list:
        return self.apply(_get_abstract)

    def __len__(self) -> int:
        return len(self.dir_dict.keys())




def download(dir: str='.') -> None:
    data = {
        'comm_use_subset':"https://ai2-semanticscholar-cord-19.s3-us-west-2.amazonaws.com/2020-03-13/comm_use_subset.tar.gz",
        'noncomm_use_subset':"https://ai2-semanticscholar-cord-19.s3-us-west-2.amazonaws.com/2020-03-13/noncomm_use_subset.tar.gz",
        'pmc_custom_license':"https://ai2-semanticscholar-cord-19.s3-us-west-2.amazonaws.com/2020-03-13/pmc_custom_license.tar.gz",
        'biorxiv_medrxiv':"https://ai2-semanticscholar-cord-19.s3-us-west-2.amazonaws.com/2020-03-13/biorxiv_medrxiv.tar.gz"
    }
    if not os.path.exists(dir):
        os.mkdir(dir)
    for d in data.keys():
        handle = urlopen(data[d])
        with open(f"{dir}/{d}.tar.gz", 'wb') as out:
            while True:
                dat = handle.read(1024)
                if len(dat) == 0: break
                out.write(dat)
    for f in os.listdir(dir):
        if tarfile.is_tarfile(f"{dir}/{f}"):
            tar = tarfile.open(f"{dir}/{f}", 'r:gz')
            tar.extractall(path=dir)
            tar.close()
            os.remove(f"{dir}/{f}")






import csv
from urllib.request import urlopen
import requests
import io

def _read_data(url):
    response = urlopen(url)
    byts = response.read()
    data = io.StringIO(byts.decode())
    reader = csv.DictReader(data)
    result = {}
    for row in reader:
        for column, value in row.items():
            result.setdefault(column, []).append(value)
    return result

def _convert_data(data: dict) -> dict:
    out = {}
    for k in data.keys():
        if k in ['Province/State', 'Country/Region']:
            out[k] = data[k]
        elif k in ['Lat', 'Long']:
            out[k] = list(map(float, data[k]))
        else:
            out[k] = list(map(int, data[k]))
    return out

def get_hopkins() -> (dict, dict, dict):
    datafiles = ["https://raw.githubusercontent.com/CSSEGISandData/COVID-19/master/csse_covid_19_data/csse_covid_19_time_series/time_series_19-covid-Confirmed.csv",
                 "https://raw.githubusercontent.com/CSSEGISandData/COVID-19/master/csse_covid_19_data/csse_covid_19_time_series/time_series_19-covid-Deaths.csv",
                 "https://raw.githubusercontent.com/CSSEGISandData/COVID-19/master/csse_covid_19_data/csse_covid_19_time_series/time_series_19-covid-Recovered.csv"]
    return (_convert_data(_read_data(dat)) for dat in datafiles)



from .data import Paperset, download
from .hopkins import get_hopkins
from .text import _get_text as text
from .text import get_texts as texts
from .text import _get_abstract as abstract
from .text import get_abstracts as abstracts



import cotools
from pprint import pprint

cotools.download(dir="data")

noncomm = cotools.Paperset("data/noncomm_use_subset")

data = cotools.Paperset("data/comm_use_subset")
pprint(data[0])
print(type(data[0]))

# get the text for one feature
cotools.text(data[0])

cotools.texts(data[:15])

data.text()

data.apply(len)

# dict

pprint(data[:2])
print(type(data[2:5]))
# list

print(len(data))

# takes about 5gb in memory
alldata = data[:]

len(data)

len(alldata)

